@@ -14,4 +14,4 @@ def serial_of(fact_id: UUID, *, fact_store: FactStore = FactStore()) -> Optional
 def publish(fact_file: Path, *, fact_store: FactStore = FactStore()) -> None:
     facts = parse_file_as(List[Fact], fact_file)
     for fact in facts:
-        fact_store.publish(fact=fact)
+        fact_store.publish(fact=fact)  # IMPR: Async or as list
