@@ -1,10 +1,11 @@
 from __future__ import annotations
+import re
 
 from typing import Type, TypeVar, Dict, List, Optional
 from uuid import uuid4
 
 from pyfactcast.grpc.generated.FactStore_pb2 import MSG_Notification
-from pydantic import BaseModel, Extra, constr
+from pydantic import BaseModel, Extra, constr, validator
 
 import json
 
@@ -15,10 +16,10 @@ class CatchUp:
     pass
 
 
-# Sadly pydantic and mypy do not play nice here.
-Id: constr = constr(  # type: ignore
-    regex=r"^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$"
+UUID4_REGEX = re.compile(
+    r"^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$"
 )
+# Sadly pydantic and mypy do not play nice here.
 AggsId: constr = constr(  # type: ignore
     regex=r"^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$"
 )
@@ -27,13 +28,21 @@ AggsId: constr = constr(  # type: ignore
 class FactHeader(BaseModel):
     class Config:
         extra = Extra.allow
+        validate_assignment = True
 
     ns: str
     type: str
-    id: Id = uuid4()  # type: ignore
+    id: str = ""  # type: ignore
     aggIds: Optional[List[AggsId]] = None  # type: ignore
     meta: Dict[str, str] = {}
 
+    @validator("id", always=True)
+    def set_id(cls, id: str) -> str:
+        if id and not UUID4_REGEX.match(id):
+            raise ValueError("id is not a valid UUID4")
+
+        return id or str(uuid4())
+
 
 class Fact(BaseModel):
     header: FactHeader
@@ -47,5 +56,10 @@ class Fact(BaseModel):
 
 
 class VersionedType(BaseModel):
-    type: str
+    name: str
     version: int = 0
+
+
+class SubscriptionSpec(BaseModel):
+    ns: str
+    type: Optional[VersionedType] = None
